//+------------------------------------------------------------------+
//|                                                     log_boll.mq5 |
//|                                  Copyright 2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 8
#property indicator_plots   4
//--- plot upper
#property indicator_label1  "upper"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrCornflowerBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot lower
#property indicator_label2  "lower"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrCornflowerBlue
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- plot base
#property indicator_label3  "base"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrCornflowerBlue
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1
//--- plot kline
#property indicator_label4  "kline"
#property indicator_type4   DRAW_COLOR_CANDLES
#property indicator_color4  clrRed,clrLime
#property indicator_style4  STYLE_SOLID
#property indicator_width4  1
//--- input parameters
input int                     Period=20;
input double                  Deviations=2;
input ENUM_APPLIED_PRICE      AppliedPrice=PRICE_CLOSE;
input int                     Shift=0;
//--- indicator buffers
double         upperBuffer[];
double         lowerBuffer[];
double         baseBuffer[];
double         klineOpen[];
double         klineHigh[];
double         klineLow[];
double         klineClose[];
double         klineColors[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,upperBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,lowerBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,baseBuffer,INDICATOR_DATA);
   SetIndexBuffer(3,klineOpen,INDICATOR_DATA);
   SetIndexBuffer(4,klineHigh,INDICATOR_DATA);
   SetIndexBuffer(5,klineLow,INDICATOR_DATA);
   SetIndexBuffer(6,klineClose,INDICATOR_DATA);
   SetIndexBuffer(7,klineColors,INDICATOR_COLOR_INDEX);
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int i = prev_calculated == 0 ? 0 : prev_calculated - 1;
   
   for (; i < rates_total; i++) {
      klineOpen[i] = MathLog10(open[i]);
      klineHigh[i] = MathLog10(high[i]);
      klineLow[i] = MathLog10(low[i]);
      klineClose[i] = MathLog10(close[i]);
      
      if (i >= Period) {
      
         baseBuffer[i] = iMa(i, klineClose);
         double std = stddev(baseBuffer[i], i, klineClose);
         
         lowerBuffer[i] = baseBuffer[i] - Deviations*std;
         upperBuffer[i] = baseBuffer[i] + Deviations*std;
         
      }
      
      klineColors[i] = (klineClose[i] >= klineOpen[i]) ? 1 : 0;
   }

   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+


double iMa(int pos, const double &price[]) {

   double sum=0;
   int start = pos - Period + 1;
   for (; start <= pos; start++) {
      sum += price[start];
   }
   
   return sum / Period;
}

double stddev(double expect, int pos, const double &price[]) {
   double sum=0;
   int start = pos - Period + 1;
   for (; start <= pos; start++) {
      sum += MathPow(price[start] - expect, 2);
   }
   
   return MathSqrt(sum / Period);
}
